library vs_flutter_cordova;

import 'dart:convert';
import 'package:webview_flutter/webview_flutter.dart';
import 'cdv_command.dart';
import 'cdv_plugin.dart';

///cordova控制器，负责管理插件和向插件分发命令
class CDVController {
  Map<String, CDVPlugin> _plugins;
  JavascriptChannel javascriptChannel;
  WebViewController webViewController;

  CDVController() {
    javascriptChannel = _cordovaJavascriptChannel();
    _plugins = {};
  }

  JavascriptChannel _cordovaJavascriptChannel() {
    return JavascriptChannel(
        name: 'flutterCordova',
        onMessageReceived: (JavascriptMessage message) {
          if (message.message.length > 0) {
            List params = jsonDecode(message.message);
            if (params.length == 4) {
              var command = new CDVCommand(
                  params[0], params[1], params[2], params[3]);
              _handleCommand((command));
            }
          }
        });
  }

  _handleCommand(CDVCommand command) {
    var plugin = _plugins[command.className];
    if (plugin != null) {
      plugin.invokeMethod(command.methodName, command.arguments, command.callbackId);
    }
  }

  void nativeCallback(String callbackId, int status, String argumentsAsJson,
      bool keepCallback, bool debug) {
    String javascript =
        "cordova.require('cordova/exec').nativeCallback('$callbackId',$status,$argumentsAsJson,$keepCallback, $debug)";
    webViewController.evaluateJavascript(javascript);
  }

  addPlugins(List<CDVPlugin> plugins) {
    for (var plugin in plugins) {
      var className = plugin.className();
      if (className != null) {
        plugin.controller = this;
        _plugins[className] = plugin;
      } else {
        throw new NullThrownError();
      }
    }
  }

  removePlugin(String className) {
    if(className != null){
      _plugins.remove(className);
    }
  }

  removePlugins(List<String> classNames) {
    if(classNames != null){
      for(var each in classNames){
        _plugins.remove(each);
      }
    }
  }
}